//
//  BonjourDiscoverSocketClient.m
//  SampleMulticast
//
//  Created by yuwei on 2011/5/3.
//  Copyright 2011年 __MyCompanyName__. All rights reserved.
//

#import "BonjourDiscoverSocketClient.h"
#import "BonjourSocketServer.h"
#import "BonjourPeer.h"

#define SERVICETYPE @"_sampleM._tcp."

@implementation BonjourDiscoverSocketClient

@synthesize netServiceBrowser;
@synthesize services;// sections;
@synthesize delegate;
@synthesize myServer;
#pragma mark - object lifecycle

- (id) init{
    self = [super init];
    if (!self) {
        return nil;
    }
    services = [[NSMutableArray alloc] init];
    return self;
}

- (void) dealloc{
    [self.netServiceBrowser stop];
    self.netServiceBrowser = nil;
    [services release];
    [super dealloc];
}

#pragma mark - service discovery

- (void)startSearchService {
    [self.netServiceBrowser stop];
    [self.services removeAllObjects];
    [delegate servicesDidChange:self];
    
    NSNetServiceBrowser *aNetServiceBrowser = [[NSNetServiceBrowser alloc] init];
    [aNetServiceBrowser setDelegate:self];
    self.netServiceBrowser = aNetServiceBrowser;
    [aNetServiceBrowser release];
    [self.netServiceBrowser searchForServicesOfType:SERVICETYPE inDomain:@""];
    
}

#pragma mark - NSNetServiceBrowser Delegate

- (void)netServiceBrowser:(NSNetServiceBrowser*)netServiceBrowser didRemoveService:(NSNetService*)service moreComing:(BOOL)moreComing {
	
    NSUInteger index = -1;
    for (BonjourPeer *peer in self.services) {
        if ([peer compare:service]) {
            index = [self.services indexOfObject:peer];
        }
    }
    if (index == -1) {
        return;
    }
    [self.services removeObjectAtIndex:index];
    
    if (!moreComing) 
        [delegate servicesDidChange:self];
}	

- (void)netServiceBrowser:(NSNetServiceBrowser*)netServiceBrowser didFindService:(NSNetService*)service moreComing:(BOOL)moreComing {

#ifndef ALLOW_TO_CONNECT_TO_SELF
	// Don't display our published record
    if (![[[UIDevice currentDevice] name] isEqualToString:[service name]]) {
        // If a service came online, add it to the list and update the table view if no more events are queued.
        BonjourPeer *newPeer = [[BonjourPeer alloc] init];
        newPeer.peer = service;
        [self.services addObject: newPeer];
        [newPeer release];
        [service setDelegate:self];
        [service resolveWithTimeout:0.0];
	}
#else
    BonjourPeer *newPeer = [[BonjourPeer alloc] init];
    newPeer.peer = service;
    [self.services addObject: newPeer];
    [newPeer release];
    [service setDelegate:self];
    [service resolveWithTimeout:0.0];
#endif
	
    
    if (!moreComing && [self.services count] > 0){
        [delegate servicesDidChange:self];
        [self.myServer performSelectorInBackground:@selector(update) withObject:nil];
    }
}	

#pragma mark - NSNetServiceDelegate Protocol

- (void)netServiceDidResolveAddress:(NSNetService *)sender {
    
    NSData *data = [sender TXTRecordData];
    NSString *message = nil;
    if (data) {
        NSDictionary *txtDict = [NSNetService dictionaryFromTXTRecordData:data];
        NSData *mData = [txtDict objectForKey:@"message"];
        if (mData) {
            message = [[NSString alloc] initWithData:mData encoding:NSUTF8StringEncoding];
        }
    }
    
    NSArray *urlArray = [message componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@";"]];
    NSString *mode = [urlArray objectAtIndex:0];
    if ([mode compare:@"E"] == NSOrderedSame) {
        for (BonjourPeer *peer in self.services) {
            if ([peer compare:sender]) {
                [peer removeAll];
            }
        }
    }
    BOOL update = YES;
    if ([mode compare:@"D"] == NSOrderedSame)
        update = NO;

    NSMutableArray *new = [NSMutableArray arrayWithArray:urlArray];
    [new removeObjectAtIndex:0];
    
    
    NSMutableSet *set = [[NSMutableSet alloc] initWithArray:new];
    for (BonjourPeer *peer in self.services) {
        if ([peer compare:sender]) {
            if (update) {
                [peer updateList:set];
            }else{
                [peer removeList:set];
            }
            
        }
    }
        
    [delegate servicesDidChange:self];
}



@end
