//
//  ModelManager.m
//  PTM
//
//  Created by yuwei on 2011/5/9.
//  Copyright 2011年 __MyCompanyName__. All rights reserved.
//


#import "ModelManager.h"
#import "Content.h"
#import "AsynNetwork.h"
#import "NetworkUtility.h"
@implementation ModelManager

@synthesize fetchedResultsController=__fetchedResultsController;

@synthesize managedObjectContext=__managedObjectContext;
@synthesize delegate, asyDelegate;
@synthesize bsServer;

#pragma mark - object lifecycle
- (id) init {
    self = [super init];
    if (!self) {
        return nil;
    }
    delegate = nil;
    return self;
}

- (void)dealloc
{
    [bsServer release];
    [__fetchedResultsController release];
    [__managedObjectContext release];
    [super dealloc];
}

#pragma mark - setter

- (void) setBsServer:(BonjourSocketServer *)newBsServer {
    if (bsServer == nil || bsServer != newBsServer) {
        [bsServer release];
        bsServer = [newBsServer retain];
        bsServer.delegate = self;
    }
    
}

#pragma mark - model manager
- (void) deleteAllObject {
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Content" inManagedObjectContext: self.managedObjectContext];
    [fetchRequest setEntity:entity];
	
    NSError *error;
    NSArray *items = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    [fetchRequest release];
    
    for(Content *content in items){
        [self.managedObjectContext deleteObject:content];
    }
    
    error = nil;
    if (![self.managedObjectContext save:&error])
    {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
         */
        abort();
    }
}

- (void) deleteObject: (NSIndexPath *) indexPath {
    NSManagedObjectContext *context = [self.fetchedResultsController managedObjectContext];    
    [context deleteObject:[self.fetchedResultsController objectAtIndexPath:indexPath]];

    NSError *error = nil;
    if (![context save:&error])
    {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
         */
        abort();
    }
}

- (Content *)findContentWithUrl:(NSString *) url{
    
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Content" inManagedObjectContext: self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
	NSPredicate *pred = [NSPredicate predicateWithFormat:@"url = %@", url];
    [fetchRequest setPredicate:pred];
    
    NSError *error;
    NSArray *items = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    [fetchRequest release];
    if ([items count] == 0) {
        return nil;
    }
    return (Content *)[items objectAtIndex:0];
}


- (Content *)insertNewObjectwithContext:(NSData *)webContext mineType:(NSString *)mineType url:(NSString *)url
{
    Content *oldContent = [self findContentWithUrl:url];
    
    if (oldContent != nil) {
        oldContent.context = webContext;
        oldContent.timeStamp = [NSDate date];
        return  oldContent;
    }
    
    
    // Create a new instance of the entity managed by the fetched results controller.
    NSManagedObjectContext *context = self.managedObjectContext;
    
    NSEntityDescription *entity = [[self.fetchedResultsController fetchRequest] entity];
    Content *newContent = [NSEntityDescription insertNewObjectForEntityForName:[entity name] inManagedObjectContext:context];
    
    
    // If appropriate, configure the new managed object.
    newContent.context = webContext;
    newContent.miniType = mineType;
    newContent.url = url;
    newContent.timeStamp = [NSDate date];
    NSNumber *rank = [[[NSNumber alloc] initWithInt:0] autorelease];
    newContent.rank = rank;
        
    // Save the context.
    NSError *error = nil;
    if (![context save:&error])
    {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
         */
        abort();
    }
    return newContent;
}



- (void)addHtmlCount:(NSURL *)url{
    NSString *urlStr = [NSString stringWithFormat:@"%@://%@%@", [url scheme], [url host],[url path]];
    Content *page = [self findContentWithUrl:urlStr];
    if (page == nil) {
        NSLog(@"error to find page to add count");
        return;
    }
    int rank =  [page.rank intValue];
    rank++;

    page.rank = [NSNumber numberWithInt:rank];
    // Save the context.
    NSError *error = nil;
    
    if (![self.managedObjectContext save:&error])
    {
        abort();
    }

}


- (Content *) contentRequest:(NSURL *)url{
    NSString *urlStr = [NSString stringWithFormat:@"%@://%@%@", [url scheme], [url host],[url path]];
    
    Content *content = [self findContentWithUrl:urlStr];
    if (content != nil) {
        return content;
    }

    AsynNetwork *download = [[AsynNetwork alloc] init];
    download.delegate = self;
   
    
    if (!downloadFromPeer) {
        NSData *newData = [[download asynDownloadData:url] retain];
        NetworkUtility *utility = [[NetworkUtility alloc] init];
        
        NSString *mineType = [utility identifyMineTypeWithNSURL:url];
        [utility release];
        NSString *urlStr = [NSString stringWithFormat:@"%@://%@%@", [url scheme], [url host], [url path]];
    
        if (newData == nil) {
            return nil;
        }
        if ([newData length] == 0) {
            return nil;
        }
        Content *d = [self insertNewObjectwithContext:newData mineType:mineType url:urlStr];
        [newData release];
        return d;
    }else{
        [asyDelegate AsynModelWillRequestContent:urlStr];
        [download asynDownloadData:url FromPeer:peer];
    }
    return nil;
}

#pragma mart - peer tp peer

- (void) usePeer:(NSNetService *)sPeer {
    peer = sPeer;
    downloadFromPeer = YES;
}

- (void) dontUsePear {
    downloadFromPeer = NO;
}

#pragma mark - AsynNetwork delegate

- (void) asynDownload: (AsynNetwork *)asynNetwork didFailWithError: (NSError *)error {

}

- (void) asynDownloaddidFinish:(AsynNetwork *)asynNetwork {
    if (asynNetwork.data == nil) {
        return;
    }
    
    NetworkUtility *utility = [[NetworkUtility alloc] init];
    NSURL *url = asynNetwork.url;    
    NSString *mineType = [utility identifyMineTypeWithNSURL:url];
    [utility release];
    NSString *urlStr = [NSString stringWithFormat:@"%@://%@%@", [url scheme], [url host], [url path]];
    Content *newContent = [self insertNewObjectwithContext:asynNetwork.data mineType:mineType url:urlStr];
    
    [asyDelegate AsynModelDidInsertContent: newContent];
    [asynNetwork autorelease];
}


#pragma mark - BonjourSocketServer Delegate

- (NSData *)respondToClient:(NSString *)url {
    Content * content = [self findContentWithUrl:url];
    if(content == nil)
        return nil;
    NSData *respond = [NSData dataWithData:content.context];
    return respond;
}

- (NSString *) cacheList {
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Content" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
        
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"miniType = 'text/html' && rank > 0"];
    [fetchRequest setPredicate:pred];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"rank" ascending:NO];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    
    [fetchRequest setSortDescriptors:sortDescriptors];

    NSError *error = nil;
    NSArray *items = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    [fetchRequest release];

    NSMutableString *txtRecord = [NSMutableString stringWithString:@"U"];
    int buffer = 254;
    for(Content *content in items){
        NSURL *url = [NSURL URLWithString:content.url];
        if(buffer > ([[url host] length] + [[url path] length] + 1)){
            buffer -= ([[url host] length] + [[url path] length] + 1);
            [txtRecord appendFormat:@";%@%@", [url host], [url path]];
        }else
            break;
    }
    return txtRecord;
}

#pragma mark - Fetched results controller

- (NSManagedObjectContext *)managedObjectContext{
    if (__managedObjectContext != nil) {
        return __managedObjectContext;
    }
    PTMAppDelegate *appDelegate = (PTMAppDelegate*)[[UIApplication sharedApplication] delegate];
    self.managedObjectContext = appDelegate.managedObjectContext;
    return __managedObjectContext;
}

- (NSFetchedResultsController *)fetchedResultsController
{
    if (__fetchedResultsController != nil)
    {
        return __fetchedResultsController;
    }
    
    /*
     Set up the fetched results controller.
     */
    // Create the fetch request for the entity.
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Content" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"miniType = 'text/html' && rank > 0"];
    [fetchRequest setPredicate:pred];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"url" ascending:NO];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:self.managedObjectContext sectionNameKeyPath:nil cacheName:nil];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
    [aFetchedResultsController release];
    [fetchRequest release];
    [sortDescriptor release];
    [sortDescriptors release];
    
	NSError *error = nil;
	if (![self.fetchedResultsController performFetch:&error])
    {
	    /*
	     Replace this implementation with code to handle the error appropriately.
         
	     abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
	     */
	    abort();
	}
    
    return __fetchedResultsController;
}    

#pragma mark - Fetched results controller delegate

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    if(delegate == nil)
        return;
    [delegate beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type
{
    if(delegate == nil)
        return;
    switch(type)
    {
        case NSFetchedResultsChangeInsert:
            [delegate inertSection: sectionIndex];
            break;
            
        case NSFetchedResultsChangeDelete:
            [delegate deleteSection: sectionIndex];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    if(delegate == nil)
        return;
    
    switch(type)
    {
            
        case NSFetchedResultsChangeInsert:
            [delegate inertRow: newIndexPath];
            break;
            
        case NSFetchedResultsChangeDelete:
            [delegate deleteRow: indexPath];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [delegate configureCell: indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [delegate deleteRow: indexPath];
            [delegate inertRow: newIndexPath];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    if(delegate != nil){
        [delegate endUpdates];
    }
    [self.bsServer update];

}



@end
