//
//  NetworkUtility.m
//  PTM
//
//  Created by yuwei on 2011/5/11.
//  Copyright 2011年 __MyCompanyName__. All rights reserved.
//

#import "NetworkUtility.h"
#import <CFNetwork/CFNetwork.h>
#import <SystemConfiguration/SystemConfiguration.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>
#include <ifaddrs.h>


@implementation NetworkUtility

#pragma mark site Reachability

// Direct from Apple. Thank you Apple
- (BOOL)addressFromString:(NSString *)IPAddress address:(struct sockaddr_in *)address
{
    if (!IPAddress || ![IPAddress length]) return NO;
	
    memset((char *) address, sizeof(struct sockaddr_in), 0);
    address->sin_family = AF_INET;
    address->sin_len = sizeof(struct sockaddr_in);
	
    int conversionResult = inet_aton([IPAddress UTF8String], &address->sin_addr);
    if (conversionResult == 0) {
		NSAssert1(conversionResult != 1, @"Failed to convert the IP address string into a sockaddr_in: %@", IPAddress);
        return NO;
    }
	
    return YES;
}

- (NSString *) getIPAddressForHost: (NSString *) theHost
{
	struct hostent *host = gethostbyname([theHost UTF8String]);
    if (!host) {herror("resolv"); return NULL; }
	struct in_addr **list = (struct in_addr **)host->h_addr_list;
	NSString *addressString = [NSString stringWithCString:inet_ntoa(*list[0]) encoding:NSUTF8StringEncoding];
	return addressString;
}

- (BOOL) hostAvailable: (NSString *) theHost
{
	if (theHost == nil) {
        return NO;
    }
    NSString *addressString = [self getIPAddressForHost:theHost];
    if (!addressString)
    {
        printf("Error recovering IP address from host name\n");
        return NO;
    }
	
    struct sockaddr_in address;
    BOOL gotAddress = [self addressFromString:addressString address:&address];
	
    if (!gotAddress)
    {
		printf("Error recovering sockaddr address from %s\n", [addressString UTF8String]);
        return NO;
    }
	
	SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&address);
    SCNetworkReachabilityFlags flags;
	
	BOOL didRetrieveFlags =SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
    CFRelease(defaultRouteReachability);
	
    if (!didRetrieveFlags)
    {
        printf("Error. Could not recover network reachability flags\n");
        return NO;
    }
	
    BOOL isReachable = flags & kSCNetworkFlagsReachable;
    return isReachable ? YES : NO;;
}

#pragma mark - MyNetwork funtionalities

- (NSString *)identifyMineTypeWithNSURL: (NSURL *)url {
    NSString *mimeType;
    
	if ([dict objectForKey:[url pathExtension]] != nil) {
		mimeType = [NSString stringWithFormat:@"%@", [dict objectForKey:[url pathExtension]]];
	}else{
		mimeType = [NSString stringWithFormat:@"text/html"];
	}
    return mimeType;
}


#pragma mark - object life cycle
- (id) init {
	self = [super init];
	if (!self) {
		return nil;
	}
	dict = [[NSDictionary alloc] initWithObjectsAndKeys:
			@"application/ad+xml", @"ad",
			@"text/css", @"css",
			@"image/gif", @"gif",
			@"text/html", @"htm",
			@"text/html", @"html",
			@"text/html", @"php",
			@"text/html", @"jsp",
			@"text/html", @"asp",
			@"text/html", @"aspx",
			@"image/jpeg", @"jpe",
			@"image/jpeg", @"jpg",
			@"image/jpeg", @"jpeg",
			@"application/x-javascript", @"js",
			@"application/pdf", @"pdf",
			@"image/png", @"png",
			@"text/plain", @"txt",
			@"image/svg+xml", @"svg",
			@"application/rss+xml", @"rss",
			nil
			];
	
	return self;
}
-(void) dealloc {
	[dict release];
	[super dealloc];
}


@end
