//
//  SocketClient.m
//  SampleMulticast
//
//  Created by yuwei on 2011/5/4.
//  Copyright 2011年 __MyCompanyName__. All rights reserved.
//

#import "SocketClient.h"
#import <sys/socket.h>
#import <netinet/in.h>
#import <unistd.h>
#import <arpa/inet.h>

@implementation SocketClient

@synthesize service, istr, ostr, delegate, urlRequest, respond;

#pragma - object lifecycle
- (id)initWithService:(NSNetService *)serviceInstance url:(NSString *)url delegate:(id<SocketClientDelegate>)anObject {
    self = [super init];
    if (!self) {
        return nil;
    }
    self.service = serviceInstance;
    self.delegate = anObject;
    self.urlRequest = url;
    [self.service getInputStream:&istr outputStream:&ostr];
	didSend = NO;
	return self;
}

-(void) dealloc {
	[istr removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[istr release];
	
	[ostr removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[ostr release];
	
	[service release];
	[super dealloc];
}

#pragma - socket connection

- (void)runConnection {

	LOGGING_FACILITY( self.istr != nil && self.ostr != nil, @"Streams not set up properly." );
	
	if (self.istr && self.ostr) {
		self.istr.delegate = self;
		[self.istr scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
		[self.istr open];
		self.ostr.delegate = self;
		[self.ostr scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
		[self.ostr open];
	}
}

- (void) sendRequest {
    NSData *data = [self.urlRequest dataUsingEncoding:NSUTF8StringEncoding];
    int len = 0;
	
	if (data) {
		len = [data length];
        len = [self.ostr write:[data bytes] maxLength:[data length]];
        [self.ostr close];
	}
}

- (void) receiveRespond {
    if (!respond) {
        respond = [[NSMutableData alloc] init];
    }
    NSInteger       bytesRead;
    uint8_t         buffer[32768];
    
    bytesRead = [(NSInputStream *)self.istr read:buffer maxLength:sizeof(buffer)];
    if (bytesRead == -1) {

    } else if (bytesRead == 0) {

    } else {
        [respond appendBytes:buffer length:bytesRead];
    }
}

#pragma - NSStream Delegate

- (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)eventCode
{
	switch(eventCode) {
		case NSStreamEventOpenCompleted:
                [delegate socketClientDidOpenConnection:self];
            break;
        case NSStreamEventHasSpaceAvailable:
            if (!didSend) {
                [self sendRequest];
                didSend = YES;
            }
		case NSStreamEventHasBytesAvailable:
			if (stream == self.istr) {
                [self receiveRespond];
			}
			break;
        case NSStreamEventEndEncountered:
            if ([self.ostr streamStatus] != NSStreamStatusOpen) {
                [delegate socketClientWillBeginReceivingData: self];
            }
            if ([self.istr streamStatus] != NSStreamStatusOpen) {
                [delegate socketClientDidFinishReceivingData: self];
            }
		default:
			break;
	}
}

@end
