//
//  SocketServerRequest.m
//  SampleMulticast
//
//  Created by yuwei on 2011/5/4.
//  Copyright 2011年 __MyCompanyName__. All rights reserved.
//

#import "SocketServerRequest.h"
#include <CFNetwork/CFSocketStream.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>

#define kMaxMessageLength 1024*1024*5

@implementation SocketServerRequest

@synthesize istr, ostr, peerName, delegate;

#pragma - object lifecycle

- (id)initWithInputStream:(NSInputStream *)readStream 
			 outputStream:(NSOutputStream *)writeStream 
					 peer:(NSString *)peerAddress 
				 delegate:(id <SocketServerRequestDelegate>)anObject {
	self = [super init];
    if (!self) {
        return nil;
    }
    self.istr = readStream;
    self.ostr = writeStream;
    self.peerName = peerAddress;
    self.delegate = anObject;

	return self;
}

- (void)dealloc {

	[istr removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[istr release];
	
	[ostr removeFromRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
	[ostr release];
	
	[peerName release];
	[super dealloc];
}

#pragma - connection

- (void)runProtocol {
	
	LOGGING_FACILITY(self.istr != nil && self.ostr != nil, @"Streams not set up properly." );
	
	if (self.istr && self.ostr) {
		self.istr.delegate = self;
		[self.istr scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
		[self.istr open];
		self.ostr.delegate = self;
		[self.ostr scheduleInRunLoop:[NSRunLoop currentRunLoop] forMode:NSRunLoopCommonModes];
		[self.ostr open];
	}
}

- (NSData *)receiveData {
    NSMutableData *request = [[NSMutableData alloc] init];
	NSInteger       bytesRead;
    uint8_t         buffer[32768];
    
    bytesRead = [self.istr read:buffer maxLength:sizeof(buffer)];
    if (bytesRead == -1) {

    } else if (bytesRead == 0) {

    } else {
        [request appendBytes:buffer length:bytesRead];
    }
	
	return [request autorelease];
}

- (void) respondRequest: (NSData *) request{
    NSString *url = [[NSString alloc] initWithData:request encoding:NSUTF8StringEncoding];
    
    NSData *context = [delegate respondToClient:url];
    
    if (context == nil) {
        [self.ostr close];
        return;
    }
    
    
    uint8_t *dataBytes = (uint8_t *)[context bytes];
    NSUInteger bytesWrittenSoFar = 0;
    do{
        NSInteger bytesWritten;
        bytesWritten = [self.ostr write:&dataBytes[bytesWrittenSoFar] maxLength:[context length] - bytesWrittenSoFar];
        if (bytesWritten != -1) {
            bytesWrittenSoFar += bytesWritten;
        }
    }while(bytesWrittenSoFar < [context length]);
    [self.ostr close];
    [url release];
}

#pragma - NSStream delegate

- (void)stream:(NSStream *)stream handleEvent:(NSStreamEvent)eventCode
{
	switch(eventCode) {
		case NSStreamEventHasSpaceAvailable:
			break;
		case NSStreamEventHasBytesAvailable:
			if (stream == self.istr) {
				NSData *request = [[self receiveData] retain];
				[self.istr close];
                if ([self.ostr hasSpaceAvailable]) {
                    [self respondRequest: request];
                    [request release];
                }
                [delegate socketServerRequestDidFinish:self];         
			}
			break;
		case NSStreamEventErrorOccurred:
			[delegate socketServerRequestDidReceiveError:self];
			break;
		default:
			break;
	}
}



@end
